package org.processmining.acceptingpetrinet.filters.impl;

import java.util.HashSet;
import java.util.Set;

import org.processmining.acceptingpetrinet.filters.StartEndFilterParameters;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

public class StartEndRemoveFilter extends StartEndAbstractFilter {

	public final static String NAME = "Remove Start/End";

	public void apply(PluginContext context, Petrinet pn, Marking initialMarking, Set<Marking> finalMarkings, 
			StartEndFilterParameters parameters) {

		Set<Transition> transitions = new HashSet<Transition>(pn.getTransitions());
		for (Transition transition : transitions) {
			if (transition.getLabel().equals(parameters.getStartLabel())) {
				int minTokens = Integer.MAX_VALUE;
				boolean ok = true;
				/*
				 * Check whether all input places have no inputs and only this
				 * transition as output.
				 */
				for (PetrinetEdge<?, ?> edge : pn.getInEdges(transition)) {
					if (edge instanceof Arc) {
						Arc arc = (Arc) edge;
						Place place = (Place) arc.getSource();
						ok = ok && pn.getInEdges(place).isEmpty() && pn.getOutEdges(place).size() == 1;
					}
				}
				if (ok) {
					for (PetrinetEdge<?, ?> edge : pn.getInEdges(transition)) {
						if (edge instanceof Arc) {
							Arc arc = (Arc) edge;
							Place place = (Place) arc.getSource();
							if (initialMarking.occurrences(place) < minTokens) {
								minTokens = initialMarking.occurrences(place);
							}
							pn.removePlace(place);
							while (initialMarking.contains(place)) {
								initialMarking.remove(place);
							}
						}
					}
					for (PetrinetEdge<?, ?> edge : pn.getOutEdges(transition)) {
						if (edge instanceof Arc) {
							Arc arc = (Arc) edge;
							Place place = (Place) arc.getTarget();
							initialMarking.add(place, minTokens);
						}
					}
					pn.removeTransition(transition);
				} else {
					transition.setInvisible(true);
				}
			} else if (transition.getLabel().equals(parameters.getEndLabel())) {
				int maxTokens = 1;
				boolean ok = true;
				for (PetrinetEdge<?, ?> edge : pn.getOutEdges(transition)) {
					if (edge instanceof Arc) {
						Arc arc = (Arc) edge;
						Place place = (Place) arc.getTarget();
						ok = ok && pn.getOutEdges(place).isEmpty() && pn.getInEdges(place).size() == 1;
					}
				}
				if (ok) {
					for (PetrinetEdge<?, ?> edge : pn.getOutEdges(transition)) {
						if (edge instanceof Arc) {
							Arc arc = (Arc) edge;
							Place place = (Place) arc.getTarget();
							if (initialMarking.occurrences(place) > maxTokens) {
								maxTokens = initialMarking.occurrences(place);
							}
							pn.removePlace(place);
						}
					}
					Marking finalMarking = new Marking();
					for (PetrinetEdge<?, ?> edge : pn.getInEdges(transition)) {
						if (edge instanceof Arc) {
							Arc arc = (Arc) edge;
							Place place = (Place) arc.getSource();
							finalMarking.add(place, maxTokens);
						}
					}
					// Set the only final marking to the minimal marking that enables this transition.
					finalMarkings.clear();
					finalMarkings.add(finalMarking);
					// Remove this transition.
					pn.removeTransition(transition);
				} else {
					transition.setInvisible(true);
				}
			}
		}
	}

	public String getName() {
		return NAME;
	}

}
