package org.processmining.acceptingpetrinet.models.impl;

import java.io.InputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.opennet.OpenNet;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnml.base.Pnml;
import org.processmining.plugins.pnml.importing.PnmlImportUtils;

public class AcceptingPetriNetFactory {

	@Deprecated
	public static AcceptingPetriNet createAcceptingPetriNet() {
		return new AcceptingPetriNetImpl();
	}

	public static AcceptingPetriNet createAcceptingPetriNet(Petrinet net) {
		return new AcceptingPetriNetImpl(net);
	}

	public static AcceptingPetriNet createAcceptingPetriNet(PluginContext context, Petrinet net) {
		return new AcceptingPetriNetImpl(context, net);
	}

	public static AcceptingPetriNet createAcceptingPetriNet(Petrinet net, Marking iMarking, Marking... fMarkings) {
		return new AcceptingPetriNetImpl(net, iMarking, fMarkings);
	}

	public static AcceptingPetriNet createAcceptingPetriNet(Petrinet net, Marking iMarking, Set<Marking> fMarkings) {
		return new AcceptingPetriNetImpl(net, iMarking, fMarkings);
	}

	public static AcceptingPetriNet importFromStream(PluginContext context, InputStream input) throws Exception {
		PnmlImportUtils utils = new PnmlImportUtils();
		Pnml pnml = utils.importPnmlFromStream(context, input, "", 0);
		if (pnml == null) {
			throw new Exception("Can not read the PNML model");
		}
		OpenNet openNet = new OpenNet(pnml.getLabel());
		Marking openInitialMarking = new Marking();
		GraphLayoutConnection openLayout = new GraphLayoutConnection(openNet);
		pnml.convertToNet(openNet, openInitialMarking, openLayout);
		Petrinet net = PetrinetFactory.newPetrinet(pnml.getLabel());
		GraphLayoutConnection layout = new GraphLayoutConnection(net);
		layout.setLabel(openLayout.getLabel());
		layout.setLayedOut(true);
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>();
		Map<Place, Place> placeMap = new HashMap<Place, Place>();
		for (Transition openTransition : openNet.getTransitions()) {
			Transition transition = net.addTransition(openTransition.getLabel());
			transitionMap.put(openTransition, transition);
			transition.setInvisible(openTransition.isInvisible());
			layout.setPosition(transition, openLayout.getPosition(openTransition));
			layout.setSize(transition, openLayout.getSize(openTransition));
		}
		for (Place openPlace : openNet.getPlaces()) {
			Place place = net.addPlace(openPlace.getLabel());
			placeMap.put(openPlace, place);
			layout.setPosition(place, openLayout.getPosition(openPlace));
			layout.setSize(place, openLayout.getSize(openPlace));
		}
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> openEdge : openNet.getEdges()) {
			PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge = null;
			if (placeMap.containsKey(openEdge.getSource())) {
				edge = net.addArc(placeMap.get(openEdge.getSource()), transitionMap.get(openEdge.getTarget()));
			} else {
				edge = net.addArc(transitionMap.get(openEdge.getSource()), placeMap.get(openEdge.getTarget()));
			}
			layout.setEdgePoints(edge, openLayout.getEdgePoints(openEdge));
		}
		context.addConnection(layout);
		Marking initialMarking = new Marking();
		for (Place place : openInitialMarking.baseSet()) {
			initialMarking.add(placeMap.get(place), openInitialMarking.occurrences(place));
		}
		context.addConnection(new InitialMarkingConnection(net, initialMarking));
		Set<Marking> finalMarkings = new HashSet<Marking>();
		for (Marking openFinalMarking : openNet.getFinalMarkings()) {
			Marking finalMarking = new Marking();
			for (Place place : openFinalMarking.baseSet()) {
				finalMarking.add(placeMap.get(place), openFinalMarking.occurrences(place));
			}
			finalMarkings.add(finalMarking);
		}
		return createAcceptingPetriNet(net, initialMarking, finalMarkings);
	}

}
