package org.processmining.acceptingpetrinet.parameters;

import java.util.HashSet;
import java.util.Set;

import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;

public class ConvertPetriNetToAcceptingPetriNetParameters extends PluginParametersImpl {

	private Marking initialMarking;
	private Set<Marking> finalMarkings;
	
	public ConvertPetriNetToAcceptingPetriNetParameters(Petrinet net) {
		super();
		Marking marking = new Marking();
		Set<Marking> markings = new HashSet<Marking>();
		for (Place place : net.getPlaces()) {
			if (net.getInEdges(place).isEmpty()) {
				marking.add(place);
			}
			if (net.getOutEdges(place).isEmpty()) {
				Marking finalMarking = new Marking();
				finalMarking.add(place);
				markings.add(finalMarking);
			}
		}
		if (markings.isEmpty()) {
			markings.add(new Marking());
		}
		setInitialMarking(marking);
		setFinalMarkings(markings);
	}
	
	public ConvertPetriNetToAcceptingPetriNetParameters(ConvertPetriNetToAcceptingPetriNetParameters parameters) {
		super(parameters);
		setInitialMarking(parameters.getInitialMarking());
		setFinalMarkings(parameters.getFinalMarkings());
	}
	
	public void setInitialMarking(Marking initialMarking) {
		this.initialMarking = initialMarking;
	}
	
	public Marking getInitialMarking() {
		return initialMarking;
	}
	
	public void setFinalMarkings(Set<Marking> finalMarkings) {
		this.finalMarkings = finalMarkings;
	}
	
	public Set<Marking> getFinalMarkings() {
		return finalMarkings;
	}
	
	public boolean equals(Object object) {
		if (object instanceof ConvertPetriNetToAcceptingPetriNetParameters) {
			ConvertPetriNetToAcceptingPetriNetParameters parameters = (ConvertPetriNetToAcceptingPetriNetParameters) object;
			return super.equals(parameters)
					&& getInitialMarking().equals(parameters.getInitialMarking()) 
					&& getFinalMarkings().equals(parameters.getFinalMarkings());
		}
		return false;
	}
}
